use crate::{err, partials, succ, Station};
use axum::{
    extract::State,
    response::{IntoResponse, Redirect},
    routing::get,
    Router,
};
use axum_extra::extract::CookieJar;
use maud::{html, Markup, PreEscaped};
use sqlx::SqlitePool;
use std::sync::Arc;
use tower_sessions::{cookie::Cookie, Session};

async fn station_picker(
    State(db): State<Arc<SqlitePool>>,
    session: Session,
    jar: CookieJar,
) -> Result<(CookieJar, Markup), (CookieJar, impl IntoResponse)> {
    let (jar, current_station_cookie) = get_station_cookie(&db, jar).await;
    if let Some(station) = current_station_cookie {
        return Err((jar, Redirect::to(&format!("/s/{}", station.id))));
    } else {
        let stations = Station::all(&db).await;
        let content = html! {
            h1 { "Wähle deine Station" }
            select onchange="window.location.href='/s/' + this.value;" {
                option selected value="" {
                    "Deine Station..."
                }
                @for station in stations {
                    option value=(station.id) { (station.name) };
                }
            }

        };
        Ok((jar, partials::page(content, session, false).await))
    }
}

async fn get_station_cookie(db: &SqlitePool, jar: CookieJar) -> (CookieJar, Option<Station>) {
    let Some(station_id) = jar.get("station_id") else {
        return (jar, None); // No station_id cookie
    };

    let station_id = match station_id.value().parse::<i64>() {
        Ok(number) => number,
        Err(_) => {
            // got some cookie which isn't a i64 -> destroy and start over again
            let jar = jar.remove(Cookie::from("station_id"));
            return (jar, None);
        }
    };

    let Some(station) = Station::find_by_id(db, station_id).await else {
        // got some cookie with an i64 which is no valid station_id -> destroy and start over again
        let jar = jar.remove(Cookie::from("station_id"));
        return (jar, None);
    };

    (jar, Some(station))
}

async fn decide_between_stations(
    cookie: &Station,
    trying_to_access: &Station,
    session: Session,
) -> Markup {
    let content = html! {
        h1 { "Wähle deine Station" }
        ul {
            li {
                "Die neu aufgerufene "
                (PreEscaped("&rarr; "))
                a href=(format!("/s/{}", trying_to_access.id)) {
                    button { (trying_to_access.name) }
                }
            }
            li {
                "Die Alte "
                (PreEscaped("&rarr; "))
                a href=(format!("/s/{}", cookie.id)) {
                    button { (cookie.name) }
                }
            }
        }
    };
    partials::page(content, session, false).await
}

async fn view(
    State(db): State<Arc<SqlitePool>>,
    session: Session,
    jar: CookieJar,
    axum::extract::Path(id): axum::extract::Path<i64>,
) -> Result<(CookieJar, Markup), (CookieJar, impl IntoResponse)> {
    let (mut jar, current_station_cookie) = get_station_cookie(&db, jar).await;
    if current_station_cookie.is_none() {
        jar = jar.add(Cookie::new("station_id", id.to_string()));
    }
    if let Some(current_station_cookie) = current_station_cookie {
        if current_station_cookie.id != id {
            // user has a cookie, which is a different station than she is trying to access
            if let Some(station) = Station::find_by_id(&db, id).await {
                jar = jar.remove(Cookie::from("station_id"));
                // trying to access valid station id
                err!(session, "Du hast versucht eine neue Station zu öffnen obwohl du bereits eine andere Station offen hattest. Welche möchtest du nun verwenden?");
                return Ok((
                    jar,
                    decide_between_stations(&current_station_cookie, &station, session).await,
                ));
            } else {
                // user trying to access _in_valid station id -> make her aware + redirect to old
                err!(session, "Du hast versucht eine Station öffnen, die es nicht gibt. Nachdem du vorher schonmal eine andere Station (die es gibt) geöffnet hattest, bist du nun zu dieser weitergeleitet worden. Wenn du das nicht willst, logg dich bitte aus.");
                return Err((
                    jar,
                    Redirect::to(&format!("/s/{}", current_station_cookie.id)),
                ));
            }
        }
    }
    let station = Station::find_by_id(&db, id).await.unwrap();
    let content = html! {
        nav {
            ul {
                li { strong { (format!("Station {}", station.name)) } }
            }
            ul {
                li { a href="/s/station-logout" { "Logout" } }
            }
        }
        h1 { "test" }
    };

    Ok((jar, partials::page(content, session, false).await))
}

async fn logout(session: Session, mut jar: CookieJar) -> (CookieJar, impl IntoResponse) {
    jar = jar.remove(Cookie::from("station_id"));

    succ!(session, "Erfolgreich ausgeloggt!");

    (jar, Redirect::to("/s"))
}

pub(super) fn routes() -> Router<Arc<SqlitePool>> {
    Router::new()
        .route("/", get(station_picker))
        .route("/{id}", get(view))
        .route("/station-logout", get(logout))
}
